"""Added source type to adviser run table

Revision ID: d77154f3c15f
Revises: 7c261487d4d4
Create Date: 2020-06-30 00:59:47.273996+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "d77154f3c15f"
down_revision = "7c261487d4d4"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    source_type = postgresql.ENUM("CLI", "KEBECHET", "S2I", "GITHUB_APP", "JUPYTER_NOTEBOOK", name="source_type")
    source_type.create(op.get_bind())
    op.add_column(
        "adviser_run",
        sa.Column(
            "source_type",
            sa.Enum("CLI", "KEBECHET", "S2I", "GITHUB_APP", "JUPYTER_NOTEBOOK", name="source_type"),
            nullable=True,
        ),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("adviser_run", "source_type")
    source_type = postgresql.ENUM("CLI", "KEBECHET", "S2I", "GITHUB_APP", "JUPYTER_NOTEBOOK", name="source_type")
    source_type.drop(op.get_bind())
    # ### end Alembic commands ###



"""Added ExternalSoftwareEnvironment, ExternalHardwareInformation, constraints for syncs

Revision ID: 9a8ce1e52efb
Revises: f60593780969
Create Date: 2019-10-07 07:40:32.070193+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "9a8ce1e52efb"
down_revision = "f60593780969"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "external_hardware_information",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("cpu_vendor", sa.Integer(), nullable=True),
        sa.Column("cpu_model", sa.Integer(), nullable=True),
        sa.Column("cpu_cores", sa.Integer(), nullable=True),
        sa.Column("cpu_model_name", sa.String(length=256), nullable=True),
        sa.Column("cpu_family", sa.Integer(), nullable=True),
        sa.Column("cpu_physical_cpus", sa.Integer(), nullable=True),
        sa.Column("gpu_model_name", sa.String(length=256), nullable=True),
        sa.Column("gpu_vendor", sa.String(length=256), nullable=True),
        sa.Column("gpu_cores", sa.Integer(), nullable=True),
        sa.Column("gpu_memory_size", sa.Integer(), nullable=True),
        sa.Column("ram_size", sa.Integer(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "external_software_environment",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("environment_name", sa.String(length=256), nullable=True),
        sa.Column("python_version", sa.String(length=256), nullable=True),
        sa.Column("image_name", sa.String(length=256), nullable=True),
        sa.Column("image_sha", sa.String(length=256), nullable=True),
        sa.Column("os_name", sa.String(length=256), nullable=True),
        sa.Column("os_version", sa.String(length=256), nullable=True),
        sa.Column("cuda_version", sa.String(length=256), nullable=True),
        sa.Column("environment_type", postgresql.ENUM("RUNTIME", "BUILDTIME", name="environment_type"), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.add_column("adviser_run", sa.Column("external_build_software_environment_id", sa.Integer(), nullable=True))
    op.add_column("adviser_run", sa.Column("external_hardware_information_id", sa.Integer(), nullable=True))
    op.add_column("adviser_run", sa.Column("external_run_software_environment_id", sa.Integer(), nullable=True))
    op.drop_constraint("adviser_run_user_build_software_environment_id_fkey", "adviser_run", type_="foreignkey")
    op.drop_constraint("adviser_run_user_run_software_environment_id_fkey", "adviser_run", type_="foreignkey")
    op.drop_constraint("adviser_run_hardware_information_id_fkey", "adviser_run", type_="foreignkey")
    op.create_foreign_key(
        None,
        "adviser_run",
        "external_software_environment",
        ["external_run_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        None,
        "adviser_run",
        "external_software_environment",
        ["external_build_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        None,
        "adviser_run",
        "external_hardware_information",
        ["external_hardware_information_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("adviser_run", "user_build_software_environment_id")
    op.drop_column("adviser_run", "user_run_software_environment_id")
    op.drop_column("adviser_run", "hardware_information_id")
    op.drop_column("hardware_information", "is_user")
    op.alter_column("python_package_version", "os_name", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.alter_column("python_package_version", "os_version", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.alter_column("python_package_version", "python_version", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.drop_column("software_environment", "is_user")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("software_environment", sa.Column("is_user", sa.BOOLEAN(), autoincrement=False, nullable=False))
    op.alter_column("python_package_version", "python_version", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.alter_column("python_package_version", "os_version", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.alter_column("python_package_version", "os_name", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.add_column("hardware_information", sa.Column("is_user", sa.BOOLEAN(), autoincrement=False, nullable=False))
    op.add_column("adviser_run", sa.Column("hardware_information_id", sa.INTEGER(), autoincrement=False, nullable=True))
    op.add_column(
        "adviser_run", sa.Column("user_run_software_environment_id", sa.INTEGER(), autoincrement=False, nullable=True)
    )
    op.add_column(
        "adviser_run", sa.Column("user_build_software_environment_id", sa.INTEGER(), autoincrement=False, nullable=True)
    )
    op.drop_constraint(None, "adviser_run", type_="foreignkey")
    op.drop_constraint(None, "adviser_run", type_="foreignkey")
    op.drop_constraint(None, "adviser_run", type_="foreignkey")
    op.create_foreign_key(
        "adviser_run_hardware_information_id_fkey",
        "adviser_run",
        "hardware_information",
        ["hardware_information_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        "adviser_run_user_run_software_environment_id_fkey",
        "adviser_run",
        "software_environment",
        ["user_run_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        "adviser_run_user_build_software_environment_id_fkey",
        "adviser_run",
        "software_environment",
        ["user_build_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("adviser_run", "external_run_software_environment_id")
    op.drop_column("adviser_run", "external_hardware_information_id")
    op.drop_column("adviser_run", "external_build_software_environment_id")
    op.drop_table("external_software_environment")
    op.drop_table("external_hardware_information")
    # ### end Alembic commands ###



"""Create index for Thoth s2i specific name and version

Revision ID: 081bee0d9587
Revises: f1824f0c1afa
Create Date: 2021-01-29 08:17:53.055331+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "081bee0d9587"
down_revision = "f1824f0c1afa"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("thoth_s2i_image_name", "software_environment", ["thoth_s2i_image_version"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("thoth_s2i_image_name", table_name="software_environment")
    # ### end Alembic commands ###



"""Make hash columns unique and add columns to Kebechet app table

Revision ID: e7ebfc165a96
Revises: 2d7ef94ff4dd
Create Date: 2020-10-15 07:39:15.406208+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "e7ebfc165a96"
down_revision = "2d7ef94ff4dd"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(None, "external_python_requirements", ["requirements_hash"])
    op.create_unique_constraint(None, "external_python_requirements_lock", ["requirements_lock_hash"])
    op.add_column(
        "kebechet_github_installations", sa.Column("advised_python_software_stack_id", sa.Integer(), nullable=True)
    )
    op.add_column(
        "kebechet_github_installations", sa.Column("external_python_software_stack_id", sa.Integer(), nullable=True)
    )
    op.add_column(
        "kebechet_github_installations", sa.Column("external_software_environment_id", sa.Integer(), nullable=True)
    )
    op.create_foreign_key(
        None,
        "kebechet_github_installations",
        "external_software_environment",
        ["external_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        None,
        "kebechet_github_installations",
        "external_python_software_stack",
        ["external_python_software_stack_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        None,
        "kebechet_github_installations",
        "python_software_stack",
        ["advised_python_software_stack_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.create_unique_constraint(None, "python_requirements", ["requirements_hash"])
    op.create_unique_constraint(None, "python_requirements_lock", ["requirements_lock_hash"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "python_requirements_lock", type_="unique")
    op.drop_constraint(None, "python_requirements", type_="unique")
    op.drop_constraint(None, "kebechet_github_installations", type_="foreignkey")
    op.drop_constraint(None, "kebechet_github_installations", type_="foreignkey")
    op.drop_constraint(None, "kebechet_github_installations", type_="foreignkey")
    op.drop_column("kebechet_github_installations", "external_software_environment_id")
    op.drop_column("kebechet_github_installations", "external_python_software_stack_id")
    op.drop_column("kebechet_github_installations", "advised_python_software_stack_id")
    op.drop_constraint(None, "external_python_requirements_lock", type_="unique")
    op.drop_constraint(None, "external_python_requirements", type_="unique")
    # ### end Alembic commands ###



"""Remove unused indexes in depends_on table

Revision ID: 3d0f03348c25
Revises: fcac91b2a3f1
Create Date: 2019-12-16 08:37:15.994395+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "3d0f03348c25"
down_revision = "fcac91b2a3f1"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("depends_on_idx", table_name="depends_on")
    op.drop_index("depends_on_version_range_idx", table_name="depends_on")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "depends_on_version_range_idx", "depends_on", ["entity_id", "version_id", "version_range"], unique=False
    )
    op.create_index("depends_on_idx", "depends_on", ["entity_id", "version_id", "version_range"], unique=False)
    # ### end Alembic commands ###



"""Create index for CVE step

Revision ID: af2b9e09e53e
Revises: 9fea7ca59f56
Create Date: 2020-01-08 13:25:27.881950+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "af2b9e09e53e"
down_revision = "9fea7ca59f56"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "has_vulnerability_python_package_version_entity_idx",
        "has_vulnerability",
        ["python_package_version_entity_id"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("has_vulnerability_python_package_version_entity_idx", table_name="has_vulnerability")
    # ### end Alembic commands ###



"""Added kebechet manager and last run columns

Revision ID: 4461784e248a
Revises: b9f960cdd223
Create Date: 2020-09-29 18:08:42.771601+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "4461784e248a"
down_revision = "b9f960cdd223"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "kebechet_github_installations", sa.Column("info_manager", sa.Boolean(), server_default="false", nullable=False)
    )
    op.add_column("kebechet_github_installations", sa.Column("last_run", sa.DateTime(), nullable=True))
    op.add_column(
        "kebechet_github_installations",
        sa.Column("pipfile_requirements_manager", sa.Boolean(), server_default="false", nullable=False),
    )
    op.add_column(
        "kebechet_github_installations",
        sa.Column("thoth_advise_manager", sa.Boolean(), server_default="false", nullable=False),
    )
    op.add_column(
        "kebechet_github_installations",
        sa.Column("thoth_provenance_manager", sa.Boolean(), server_default="false", nullable=False),
    )
    op.add_column(
        "kebechet_github_installations",
        sa.Column("update_manager", sa.Boolean(), server_default="false", nullable=False),
    )
    op.add_column(
        "kebechet_github_installations",
        sa.Column("version_manager", sa.Boolean(), server_default="false", nullable=False),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("kebechet_github_installations", "version_manager")
    op.drop_column("kebechet_github_installations", "update_manager")
    op.drop_column("kebechet_github_installations", "thoth_provenance_manager")
    op.drop_column("kebechet_github_installations", "thoth_advise_manager")
    op.drop_column("kebechet_github_installations", "pipfile_requirements_manager")
    op.drop_column("kebechet_github_installations", "last_run")
    op.drop_column("kebechet_github_installations", "info_manager")
    # ### end Alembic commands ###



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2021 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Adapter for accessing Argo Workflow logs."""

import os
from typing import Optional

from thoth.storages.base import StorageBase
from thoth.storages.ceph import CephStore
from thoth.storages.exceptions import MultipleFoundError
from thoth.storages.exceptions import NotFoundError


class WorkflowLogsStore(StorageBase):
    """Access logs stored by Argo Workflows."""

    def __init__(
        self,
        deployment_name: Optional[str] = None,
        *,
        host: Optional[str] = None,
        key_id: Optional[str] = None,
        secret_key: Optional[str] = None,
        bucket: Optional[str] = None,
        region: Optional[str] = None,
        prefix: Optional[str] = None,
    ):
        """Initialize the adapter."""
        self.deployment_name = deployment_name or os.environ["THOTH_DEPLOYMENT_NAME"]
        self.prefix = "{}/{}/argo/artifacts".format(
            prefix or os.environ["THOTH_CEPH_BUCKET_PREFIX"],
            self.deployment_name,
        )
        self.ceph = CephStore(
            self.prefix, host=host, key_id=key_id, secret_key=secret_key, bucket=bucket, region=region
        )

    def get_log(self, workflow_id: str) -> str:
        """Obtain log from the given workflow."""
        results = list(self.ceph.get_document_listing(workflow_id))
        if len(results) > 1:
            raise MultipleFoundError(
                f"Multiple results match the given workflow_id ({workflow_id!r}) provided: {results!r}"
            )

        # Make sure users do not use workflow id prefix.
        if not results or not results[0].startswith(f"{workflow_id}/"):
            raise NotFoundError(f"No log entry found for {workflow_id!r}")

        return self.ceph.retrieve_blob(results[0]).decode()

    def connect(self) -> None:
        """Connect to Ceph."""
        self.ceph.connect()



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2020 Kevin Postlethwait, Francesco Murdaca
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Adapter for persisting Security Indicator results."""

import os
import logging
from typing import Any
from typing import Dict
from typing import Optional
from typing import Generator

from .ceph import CephStore

_LOGGER = logging.getLogger(__name__)


def _get_security_indicators_prefix(security_indicator_id: Optional[str] = None) -> str:
    """Get prefix where security indicators are stored."""
    bucket_prefix = os.environ["THOTH_CEPH_BUCKET_PREFIX"]
    deployment_name = os.environ["THOTH_DEPLOYMENT_NAME"]

    if security_indicator_id is None:
        return f"{bucket_prefix}/{deployment_name}/security-indicators"

    return f"{bucket_prefix}/{deployment_name}/security-indicators/{security_indicator_id}"


class _SecurityIndicatorBase:
    """A base class for security-indicators analyzers results."""

    __slots__ = ["ceph", "security_indicator_id"]

    def connect(self) -> None:
        """Connect this adapter to Ceph."""
        self.ceph.connect()

    def is_connected(self) -> bool:
        """Check if this adapter is connected."""
        return self.ceph.is_connected()

    def check_connection(self) -> None:
        """Check connection of this adapter."""
        return self.ceph.check_connection()


class SIBanditStore(_SecurityIndicatorBase):
    """An adapter for manipulating with security-indicators bandit."""

    def __init__(self, security_indicator_id: str) -> None:
        """Constructor."""
        prefix = f"{_get_security_indicators_prefix(security_indicator_id)}/"
        self.ceph = CephStore(prefix=prefix)
        self.security_indicator_id = security_indicator_id

    def retrieve_document(self) -> Dict[str, Any]:
        """Retrieve SI bandit document."""
        return self.ceph.retrieve_document("bandit")

    def document_exists(self) -> bool:
        """Check if the there is an object with the given key in bucket."""
        return self.ceph.document_exists("bandit")


class SIClocStore(_SecurityIndicatorBase):
    """An adapter for manipulating with security-indicators cloc."""

    def __init__(self, security_indicator_id: str) -> None:
        """Constructor."""
        prefix = f"{_get_security_indicators_prefix(security_indicator_id)}/"
        self.ceph = CephStore(prefix=prefix)
        self.security_indicator_id = security_indicator_id

    def retrieve_document(self) -> Dict[str, Any]:
        """Retrieve SI cloc document."""
        return self.ceph.retrieve_document("cloc")

    def document_exists(self) -> bool:
        """Check if the there is an object with the given key in bucket."""
        return self.ceph.document_exists("cloc")


class SIAggregatedStore(_SecurityIndicatorBase):
    """An adapter for manipulating with security-indicators aggregated."""

    def __init__(self, security_indicator_id: str) -> None:
        """Constructor."""
        prefix = f"{_get_security_indicators_prefix(security_indicator_id)}/"
        self.ceph = CephStore(prefix=prefix)
        self.security_indicator_id = security_indicator_id

    def retrieve_document(self) -> Dict[str, Any]:
        """Retrieve SI aggregated document."""
        return self.ceph.retrieve_document("aggregated")

    def document_exists(self) -> bool:
        """Check if the there is an object with the given key in bucket."""
        return self.ceph.document_exists("aggregated")


class SecurityIndicatorsResultsStore:
    """Adapter for manipulating with Security Indicators."""

    __slots__ = ["bandit", "cloc", "aggregated", "security_indicator_id"]

    def __init__(self, security_indicator_id: str) -> None:
        """A representation of a Security Indicator."""
        self.security_indicator_id = security_indicator_id
        self.bandit = SIBanditStore(security_indicator_id)
        self.cloc = SIClocStore(security_indicator_id)
        self.aggregated = SIAggregatedStore(security_indicator_id)

    def connect(self) -> None:
        """Connect this adapter."""
        self.bandit.connect()
        self.cloc.connect()
        self.aggregated.connect()

    def is_connected(self) -> bool:
        """Check if this adapter is connected."""
        return self.bandit.is_connected() and self.cloc.is_connected() and self.aggregated.is_connected()

    def check_connection(self):
        """Check connections of this adapter."""
        self.bandit.check_connection()
        self.cloc.check_connection()
        self.aggregated.check_connection()

    @classmethod
    def iter_security_indicators(cls) -> Generator[str, None, None]:
        """Iterate over security_indicators ids stored."""
        ceph = CephStore(prefix=_get_security_indicators_prefix())
        ceph.connect()

        last_id = None
        for item in ceph.get_document_listing():
            security_indicator_id = item.split("/", maxsplit=1)[0]
            if last_id == security_indicator_id:
                # Return only unique si ids, discard any results placed under the given prefix.
                continue

            last_id = security_indicator_id
            yield security_indicator_id

    @classmethod
    def get_security_indicators_count(cls) -> int:
        """Get number of security_indicators stored."""
        return sum(1 for _ in cls.iter_security_indicators())



"""Added PythonPackageMetadata table

Revision ID: e05d6d95ea95
Revises: 74f68dabbb46
Create Date: 2019-10-15 16:59:27.902479+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "e05d6d95ea95"
down_revision = "74f68dabbb46"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "python_package_metadata",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("author", sa.String(length=256), nullable=True),
        sa.Column("author_email", sa.String(length=256), nullable=True),
        sa.Column("classifier", sa.String(length=256), nullable=True),
        sa.Column("download_url", sa.String(length=256), nullable=True),
        sa.Column("home_page", sa.String(length=256), nullable=True),
        sa.Column("keywords", sa.String(length=256), nullable=True),
        sa.Column("license", sa.String(length=256), nullable=True),
        sa.Column("maintainer", sa.String(length=256), nullable=True),
        sa.Column("maintainer_email", sa.String(length=256), nullable=True),
        sa.Column("metadata_version", sa.String(length=256), nullable=True),
        sa.Column("name", sa.String(length=256), nullable=True),
        sa.Column("platform", sa.String(length=256), nullable=True),
        sa.Column("requires_dist", sa.String(length=256), nullable=True),
        sa.Column("summary", sa.String(length=256), nullable=True),
        sa.Column("version", sa.String(length=256), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.add_column("python_package_version", sa.Column("python_package_metadata_id", sa.Integer(), nullable=True))
    op.create_foreign_key(
        None,
        "python_package_version",
        "python_package_metadata",
        ["python_package_metadata_id"],
        ["id"],
        ondelete="CASCADE",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "python_package_version", type_="foreignkey")
    op.drop_column("python_package_version", "python_package_metadata_id")
    op.drop_table("python_package_metadata")
    # ### end Alembic commands ###



"""Robust Syncs

Revision ID: 650a13fd0a08
Revises: 9a8ce1e52efb
Create Date: 2019-10-07 14:47:33.441781+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "650a13fd0a08"
down_revision = "9a8ce1e52efb"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "external_python_requirements_lock",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False),
        sa.Column("python_software_stack_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["python_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_package_version_entity_id", "python_software_stack_id"),
    )
    op.add_column("identified", sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False))
    op.drop_constraint("identified_python_package_version_id_fkey", "identified", type_="foreignkey")
    op.create_foreign_key(
        None,
        "identified",
        "python_package_version_entity",
        ["python_package_version_entity_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("identified", "python_package_version_id")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "identified", sa.Column("python_package_version_id", sa.INTEGER(), autoincrement=False, nullable=False)
    )
    op.drop_constraint(None, "identified", type_="foreignkey")
    op.create_foreign_key(
        "identified_python_package_version_id_fkey",
        "identified",
        "python_package_version",
        ["python_package_version_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("identified", "python_package_version_entity_id")
    op.drop_table("external_python_requirements_lock")
    # ### end Alembic commands ###



"""Remove platform from Depends On

Revision ID: 894ddaef0c43
Revises: 50081eea5da2
Create Date: 2021-06-16 07:52:16.338937+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "894ddaef0c43"
down_revision = "50081eea5da2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("depends_on_platform", table_name="depends_on")
    op.drop_column("depends_on", "platform")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "depends_on",
        sa.Column(
            "platform", sa.TEXT(), server_default=sa.text("'linux-x86_64'::text"), autoincrement=False, nullable=False
        ),
    )
    op.create_index("depends_on_platform", "depends_on", ["platform"], unique=False)
    # ### end Alembic commands ###



"""Add container image environment variables

Revision ID: cb86ec08e5ce
Revises: 081bee0d9587
Create Date: 2021-03-09 11:39:07.315261+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "cb86ec08e5ce"
down_revision = "081bee0d9587"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("external_software_environment", sa.Column("env_image_name", sa.Text(), nullable=True))
    op.add_column("external_software_environment", sa.Column("env_image_tag", sa.Text(), nullable=True))
    op.add_column("software_environment", sa.Column("env_image_name", sa.Text(), nullable=True))
    op.add_column("software_environment", sa.Column("env_image_tag", sa.Text(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("software_environment", "env_image_tag")
    op.drop_column("software_environment", "env_image_name")
    op.drop_column("external_software_environment", "env_image_tag")
    op.drop_column("external_software_environment", "env_image_name")
    # ### end Alembic commands ###



"""Use Text instead of Strings all over the place

Revision ID: 0ea53ec16d6b
Revises: 125a71b49a50
Create Date: 2020-03-25 08:21:13.153294+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "0ea53ec16d6b"
down_revision = "125a71b49a50"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "adviser_run",
        "adviser_document_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "adviser_run", "adviser_name", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "adviser_run", "adviser_version", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "adviser_run", "origin", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "adviser_run",
        "re_run_adviser_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column("cve", "advisory", existing_type=sa.VARCHAR(length=16384), type_=sa.Text(), existing_nullable=True)
    op.alter_column("cve", "cve_id", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False)
    op.alter_column("cve", "cve_name", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True)
    op.alter_column(
        "cve", "version_range", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "deb_dependency", "package_name", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "deb_depends", "version_range", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "deb_package_version", "arch", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "deb_package_version", "epoch", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "deb_package_version",
        "package_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "deb_package_version",
        "package_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "deb_pre_depends",
        "version_range",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "deb_replaces", "version_range", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "dependency_monkey_run",
        "decision",
        existing_type=sa.VARCHAR(length=64),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "dependency_monkey_run",
        "dependency_monkey_document_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "dependency_monkey_run",
        "dependency_monkey_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "dependency_monkey_run",
        "dependency_monkey_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "depends_on", "version_range", existing_type=sa.VARCHAR(length=128), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "ecosystem_solver", "ecosystem", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "ecosystem_solver", "os_name", existing_type=sa.VARCHAR(length=128), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "ecosystem_solver", "os_version", existing_type=sa.VARCHAR(length=8), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "ecosystem_solver",
        "python_version",
        existing_type=sa.VARCHAR(length=8),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "ecosystem_solver",
        "solver_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "ecosystem_solver",
        "solver_version",
        existing_type=sa.VARCHAR(length=16),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "external_hardware_information",
        "cpu_model_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_hardware_information",
        "gpu_model_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_hardware_information",
        "gpu_vendor",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "cuda_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "environment_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "image_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "image_sha",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "os_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "os_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "python_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "found_python_file", "file", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "hardware_information",
        "cpu_model_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "hardware_information",
        "gpu_model_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "hardware_information",
        "gpu_vendor",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "included_file", "file", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "inspection_run", "amun_version", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "inspection_run",
        "inspection_document_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "package_analyzer_run",
        "package_analysis_document_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "package_analyzer_run",
        "package_analyzer_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "package_analyzer_run",
        "package_analyzer_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "package_extract_run",
        "analysis_document_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run",
        "image_tag",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run", "origin", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "package_extract_run", "os_id", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "package_extract_run", "os_name", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "package_extract_run",
        "os_version_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run",
        "package_extract_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run",
        "package_extract_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "origin",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "provenance_checker_run",
        "provenance_checker_document_id",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "provenance_checker_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "provenance_checker_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_artifact",
        "artifact_hash_sha256",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_artifact",
        "artifact_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "python_file_digest", "sha256", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "python_interpreter", "link", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "python_interpreter", "path", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "python_interpreter", "version", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "python_package_index", "url", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "python_package_index",
        "warehouse_api_url",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "python_package_requirement",
        "name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_requirement",
        "version_range",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "os_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "os_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "package_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "package_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "python_package_version",
        "python_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version_entity",
        "package_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version_entity",
        "package_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "rpm_package_version", "arch", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "rpm_package_version", "epoch", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "rpm_package_version",
        "package_identifier",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version",
        "package_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version",
        "package_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version", "release", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "rpm_requirement",
        "rpm_requirement_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "software_environment",
        "cuda_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "environment_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "image_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "image_sha",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment", "os_name", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column(
        "software_environment",
        "os_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "python_version",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=True,
    )
    op.alter_column(
        "solved", "document_id", existing_type=sa.VARCHAR(length=128), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "versioned_symbol",
        "library_name",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Text(),
        existing_nullable=False,
    )
    op.alter_column(
        "versioned_symbol", "symbol", existing_type=sa.VARCHAR(length=256), type_=sa.Text(), existing_nullable=False
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "versioned_symbol", "symbol", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "versioned_symbol",
        "library_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "solved", "document_id", existing_type=sa.Text(), type_=sa.VARCHAR(length=128), existing_nullable=False
    )
    op.alter_column(
        "software_environment",
        "python_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "os_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment", "os_name", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "software_environment",
        "image_sha",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "image_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "environment_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "software_environment",
        "cuda_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "rpm_requirement",
        "rpm_requirement_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version", "release", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "rpm_package_version",
        "package_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version",
        "package_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version",
        "package_identifier",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "rpm_package_version", "epoch", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "rpm_package_version", "arch", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "python_package_version_entity",
        "package_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "python_package_version_entity",
        "package_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "python_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "package_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "python_package_version",
        "package_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "os_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_version",
        "os_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_requirement",
        "version_range",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_requirement",
        "name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "python_package_index",
        "warehouse_api_url",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "python_package_index", "url", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "python_interpreter", "version", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "python_interpreter", "path", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "python_interpreter", "link", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "python_file_digest", "sha256", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "python_artifact",
        "artifact_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "python_artifact",
        "artifact_hash_sha256",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "provenance_checker_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "provenance_checker_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "provenance_checker_document_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "provenance_checker_run",
        "origin",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "package_extract_run",
        "package_extract_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run",
        "package_extract_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run",
        "os_version_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run", "os_name", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "package_extract_run", "os_id", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "package_extract_run", "origin", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "package_extract_run",
        "image_tag",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "package_extract_run",
        "analysis_document_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "package_analyzer_run",
        "package_analyzer_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "package_analyzer_run",
        "package_analyzer_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "package_analyzer_run",
        "package_analysis_document_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "inspection_run",
        "inspection_document_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "inspection_run", "amun_version", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "included_file", "file", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "hardware_information",
        "gpu_vendor",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "hardware_information",
        "gpu_model_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "hardware_information",
        "cpu_model_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "found_python_file", "file", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "external_software_environment",
        "python_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "os_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "os_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "image_sha",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "image_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "environment_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_software_environment",
        "cuda_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_hardware_information",
        "gpu_vendor",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_hardware_information",
        "gpu_model_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "external_hardware_information",
        "cpu_model_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "ecosystem_solver",
        "solver_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=16),
        existing_nullable=False,
    )
    op.alter_column(
        "ecosystem_solver",
        "solver_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "ecosystem_solver",
        "python_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=8),
        existing_nullable=False,
    )
    op.alter_column(
        "ecosystem_solver", "os_version", existing_type=sa.Text(), type_=sa.VARCHAR(length=8), existing_nullable=False
    )
    op.alter_column(
        "ecosystem_solver", "os_name", existing_type=sa.Text(), type_=sa.VARCHAR(length=128), existing_nullable=False
    )
    op.alter_column(
        "ecosystem_solver", "ecosystem", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "depends_on", "version_range", existing_type=sa.Text(), type_=sa.VARCHAR(length=128), existing_nullable=True
    )
    op.alter_column(
        "dependency_monkey_run",
        "dependency_monkey_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "dependency_monkey_run",
        "dependency_monkey_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "dependency_monkey_run",
        "dependency_monkey_document_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "dependency_monkey_run",
        "decision",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=64),
        existing_nullable=False,
    )
    op.alter_column(
        "deb_replaces", "version_range", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "deb_pre_depends",
        "version_range",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "deb_package_version",
        "package_version",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "deb_package_version",
        "package_name",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    op.alter_column(
        "deb_package_version", "epoch", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "deb_package_version", "arch", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "deb_depends", "version_range", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "deb_dependency", "package_name", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "cve", "version_range", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column("cve", "cve_name", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True)
    op.alter_column("cve", "cve_id", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False)
    op.alter_column("cve", "advisory", existing_type=sa.Text(), type_=sa.VARCHAR(length=16384), existing_nullable=True)
    op.alter_column(
        "adviser_run",
        "re_run_adviser_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=True,
    )
    op.alter_column(
        "adviser_run", "origin", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=True
    )
    op.alter_column(
        "adviser_run", "adviser_version", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "adviser_run", "adviser_name", existing_type=sa.Text(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "adviser_run",
        "adviser_document_id",
        existing_type=sa.Text(),
        type_=sa.VARCHAR(length=256),
        existing_nullable=False,
    )
    # ### end Alembic commands ###



"""Introduce PI PyBench for Python Interpreter

Revision ID: f00fc22d1589
Revises: 3d0f03348c25
Create Date: 2019-12-16 12:40:36.344874+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "f00fc22d1589"
down_revision = "3d0f03348c25"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "pi_pybench",
        sa.Column("component", sa.String(length=256), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=False),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("rounds", sa.Integer(), nullable=False),
        sa.Column("built_in_function_calls_average", sa.Float(), nullable=False),
        sa.Column("built_in_method_lookup_average", sa.Float(), nullable=False),
        sa.Column("compare_floats_average", sa.Float(), nullable=False),
        sa.Column("compare_floats_integers_average", sa.Float(), nullable=False),
        sa.Column("compare_integers_average", sa.Float(), nullable=False),
        sa.Column("compare_interned_strings_average", sa.Float(), nullable=False),
        sa.Column("compare_longs_average", sa.Float(), nullable=False),
        sa.Column("compare_strings_average", sa.Float(), nullable=False),
        sa.Column("compare_unicode_average", sa.Float(), nullable=False),
        sa.Column("concat_strings_average", sa.Float(), nullable=False),
        sa.Column("concat_unicode_average", sa.Float(), nullable=False),
        sa.Column("create_instances_average", sa.Float(), nullable=False),
        sa.Column("create_new_instances_average", sa.Float(), nullable=False),
        sa.Column("create_strings_with_concat_average", sa.Float(), nullable=False),
        sa.Column("create_unicode_with_concat_average", sa.Float(), nullable=False),
        sa.Column("dict_creation_average", sa.Float(), nullable=False),
        sa.Column("dict_with_float_keys_average", sa.Float(), nullable=False),
        sa.Column("dict_with_integer_keys_average", sa.Float(), nullable=False),
        sa.Column("dict_with_string_keys_average", sa.Float(), nullable=False),
        sa.Column("for_loops_average", sa.Float(), nullable=False),
        sa.Column("if_then_else_average", sa.Float(), nullable=False),
        sa.Column("list_slicing_average", sa.Float(), nullable=False),
        sa.Column("nested_for_loops_average", sa.Float(), nullable=False),
        sa.Column("normal_class_attribute_average", sa.Float(), nullable=False),
        sa.Column("normal_instance_attribute_average", sa.Float(), nullable=False),
        sa.Column("python_function_calls_average", sa.Float(), nullable=False),
        sa.Column("python_method_calls_average", sa.Float(), nullable=False),
        sa.Column("recursion_average", sa.Float(), nullable=False),
        sa.Column("second_import_average", sa.Float(), nullable=False),
        sa.Column("second_package_import_average", sa.Float(), nullable=False),
        sa.Column("second_submodule_import_average", sa.Float(), nullable=False),
        sa.Column("simple_complex_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_dict_manipulation_average", sa.Float(), nullable=False),
        sa.Column("simple_float_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_int_float_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_integer_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_list_manipulation_average", sa.Float(), nullable=False),
        sa.Column("simple_long_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("small_lists_average", sa.Float(), nullable=False),
        sa.Column("small_tuples_average", sa.Float(), nullable=False),
        sa.Column("special_class_attribute_average", sa.Float(), nullable=False),
        sa.Column("special_instance_attribute_average", sa.Float(), nullable=False),
        sa.Column("string_mappings_average", sa.Float(), nullable=False),
        sa.Column("string_predicates_average", sa.Float(), nullable=False),
        sa.Column("string_slicing_average", sa.Float(), nullable=False),
        sa.Column("try_except_average", sa.Float(), nullable=False),
        sa.Column("try_raise_except_average", sa.Float(), nullable=False),
        sa.Column("tuple_slicing_average", sa.Float(), nullable=False),
        sa.Column("unicode_mappings_average", sa.Float(), nullable=False),
        sa.Column("unicode_predicates_average", sa.Float(), nullable=False),
        sa.Column("unicode_properties_average", sa.Float(), nullable=False),
        sa.Column("unicode_slicing_average", sa.Float(), nullable=False),
        sa.Column("totals_average", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.add_column("pi_conv1d", sa.Column("component", sa.String(length=256), nullable=False))
    op.alter_column("pi_conv1d", "origin", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.drop_column("pi_conv1d", "framework")
    op.add_column("pi_conv2d", sa.Column("component", sa.String(length=256), nullable=False))
    op.alter_column("pi_conv2d", "origin", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.drop_column("pi_conv2d", "framework")
    op.add_column("pi_matmul", sa.Column("component", sa.String(length=256), nullable=False))
    op.alter_column("pi_matmul", "origin", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.drop_column("pi_matmul", "framework")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("pi_matmul", sa.Column("framework", sa.VARCHAR(length=256), autoincrement=False, nullable=True))
    op.alter_column("pi_matmul", "origin", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.drop_column("pi_matmul", "component")
    op.add_column("pi_conv2d", sa.Column("framework", sa.VARCHAR(length=256), autoincrement=False, nullable=True))
    op.alter_column("pi_conv2d", "origin", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.drop_column("pi_conv2d", "component")
    op.add_column("pi_conv1d", sa.Column("framework", sa.VARCHAR(length=256), autoincrement=False, nullable=True))
    op.alter_column("pi_conv1d", "origin", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.drop_column("pi_conv1d", "component")
    op.drop_table("pi_pybench")
    # ### end Alembic commands ###



"""Relax constraints for ExternalSoftwareEnvironment and SoftwareEnvironment for PackageExtractRun

Revision ID: 331cbc965699
Revises: 079e1d92f9d4
Create Date: 2019-10-08 15:51:15.249467+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "331cbc965699"
down_revision = "079e1d92f9d4"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "package_extract_run", "external_software_environment_id", existing_type=sa.INTEGER(), nullable=True
    )
    op.alter_column("package_extract_run", "software_environment_id", existing_type=sa.INTEGER(), nullable=True)
    op.alter_column("python_package_version", "package_version", existing_type=sa.VARCHAR(length=256), nullable=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("python_package_version", "package_version", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.alter_column("package_extract_run", "software_environment_id", existing_type=sa.INTEGER(), nullable=False)
    op.alter_column(
        "package_extract_run", "external_software_environment_id", existing_type=sa.INTEGER(), nullable=False
    )
    # ### end Alembic commands ###



"""Add performance and security enums for recommendation types

Revision ID: 83e2900c3721
Revises: d6d0b20ec650
Create Date: 2020-08-19 06:49:24.347264+00:00

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = "83e2900c3721"
down_revision = "d6d0b20ec650"
branch_labels = None
depends_on = None

# Solution based on https://stackoverflow.com/a/33617845

name = "recommendation_type"
tmp_name = "tmp_" + name

old_options = ("STABLE", "TESTING", "LATEST")
new_options = sorted(old_options + ("PERFORMANCE", "SECURITY"))

new_type = sa.Enum(*new_options, name=name)
old_type = sa.Enum(*old_options, name=name)

tcr = sa.sql.table("adviser_run", sa.Column("recommendation_type", new_type, nullable=False))


def upgrade():
    op.execute("ALTER TYPE " + name + " RENAME TO " + tmp_name)

    new_type.create(op.get_bind())
    op.execute(
        "ALTER TABLE adviser_run ALTER COLUMN recommendation_type "
        + "TYPE "
        + name
        + " USING recommendation_type::text::"
        + name
    )
    op.execute("DROP TYPE " + tmp_name)


def downgrade():
    # Convert 'performance' recommendation type into 'stable'
    # Convert 'security' recommendation type into 'stable'
    op.execute(tcr.update().where(tcr.c.recommendation_type == "PERFORMANCE").values(recommendation_type="STABLE"))
    op.execute(tcr.update().where(tcr.c.recommendation_type == "SECURITY").values(recommendation_type="STABLE"))

    op.execute("ALTER TYPE " + name + " RENAME TO " + tmp_name)

    old_type.create(op.get_bind())
    op.execute(
        "ALTER TABLE adviser_run ALTER COLUMN recommendation_type "
        + "TYPE "
        + name
        + " USING recommendation_type::text::"
        + name
    )
    op.execute("DROP TYPE " + tmp_name)



"""Artifact should link just to entity

Revision ID: 184e698fe2c2
Revises: a8a4748e85cb
Create Date: 2019-10-10 09:31:41.630561+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "184e698fe2c2"
down_revision = "a8a4748e85cb"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("has_artifact_python_package_version_id_fkey", "has_artifact", type_="foreignkey")
    op.drop_column("has_artifact", "python_package_version_id")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "has_artifact", sa.Column("python_package_version_id", sa.INTEGER(), autoincrement=False, nullable=False)
    )
    op.create_foreign_key(
        "has_artifact_python_package_version_id_fkey",
        "has_artifact",
        "python_package_version",
        ["python_package_version_id"],
        ["id"],
        ondelete="CASCADE",
    )
    # ### end Alembic commands ###



"""Remove package-analyzer

Revision ID: 6389973bd8b6
Revises: 84588707e6ba
Create Date: 2020-09-14 08:31:09.089734+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "6389973bd8b6"
down_revision = "84588707e6ba"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("investigated")
    op.drop_table("investigated_file")
    op.drop_table("package_analyzer_run")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "investigated_file",
        sa.Column("package_analyzer_run_id", sa.INTEGER(), autoincrement=False, nullable=False),
        sa.Column("python_file_digest_id", sa.INTEGER(), autoincrement=False, nullable=False),
        sa.ForeignKeyConstraint(
            ["package_analyzer_run_id"],
            ["package_analyzer_run.id"],
            name="investigated_file_package_analyzer_run_id_fkey",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["python_file_digest_id"],
            ["python_file_digest.id"],
            name="investigated_file_python_file_digest_id_fkey",
            ondelete="CASCADE",
        ),
    )
    op.create_table(
        "package_analyzer_run",
        sa.Column(
            "id",
            sa.INTEGER(),
            server_default=sa.text("nextval('package_analyzer_run_id_seq'::regclass)"),
            autoincrement=True,
            nullable=False,
        ),
        sa.Column("package_analyzer_name", sa.TEXT(), autoincrement=False, nullable=True),
        sa.Column("package_analyzer_version", sa.TEXT(), autoincrement=False, nullable=True),
        sa.Column("package_analysis_document_id", sa.TEXT(), autoincrement=False, nullable=False),
        sa.Column("datetime", postgresql.TIMESTAMP(), autoincrement=False, nullable=False),
        sa.Column("debug", sa.BOOLEAN(), autoincrement=False, nullable=False),
        sa.Column("package_analyzer_error", sa.BOOLEAN(), autoincrement=False, nullable=False),
        sa.Column("duration", sa.INTEGER(), autoincrement=False, nullable=True),
        sa.Column("input_python_package_version_entity_id", sa.INTEGER(), autoincrement=False, nullable=True),
        sa.ForeignKeyConstraint(
            ["input_python_package_version_entity_id"],
            ["python_package_version_entity.id"],
            name="package_analyzer_run_input_python_package_version_entity_i_fkey",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("id", name="package_analyzer_run_pkey"),
        postgresql_ignore_search_path=False,
    )
    op.create_table(
        "investigated",
        sa.Column("package_analyzer_run_id", sa.INTEGER(), autoincrement=False, nullable=False),
        sa.Column("python_artifact_id", sa.INTEGER(), autoincrement=False, nullable=False),
        sa.ForeignKeyConstraint(
            ["package_analyzer_run_id"],
            ["package_analyzer_run.id"],
            name="investigated_package_analyzer_run_id_fkey",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["python_artifact_id"],
            ["python_artifact.id"],
            name="investigated_python_artifact_id_fkey",
            ondelete="CASCADE",
        ),
    )
    # ### end Alembic commands ###



"""Adjust types for conv PI

Revision ID: 1fbd52a03c73
Revises: d0ebc816243c
Create Date: 2020-01-10 18:35:09.115584+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "1fbd52a03c73"
down_revision = "d0ebc816243c"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "pi_conv1d",
        "padding",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Integer(),
        existing_nullable=False,
        postgresql_using="padding::integer",
    )
    op.alter_column(
        "pi_conv2d",
        "padding",
        existing_type=sa.VARCHAR(length=256),
        type_=sa.Integer(),
        existing_nullable=False,
        postgresql_using="padding::integer",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "pi_conv2d", "padding", existing_type=sa.Integer(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    op.alter_column(
        "pi_conv1d", "padding", existing_type=sa.Integer(), type_=sa.VARCHAR(length=256), existing_nullable=False
    )
    # ### end Alembic commands ###



"""Add provides extra column to Python metadata

Revision ID: bcf07d69fc15
Revises: ff18da87f7c1
Create Date: 2019-10-22 11:21:47.015295+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "bcf07d69fc15"
down_revision = "ff18da87f7c1"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("python_package_metadata", sa.Column("provides_extra", sa.String(length=256), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("python_package_metadata", "provides_extra")
    # ### end Alembic commands ###



"""Create Python package entity rules

Revision ID: db6290ac490d
Revises: abb1a0a90885
Create Date: 2021-06-01 08:33:48.909928+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "db6290ac490d"
down_revision = "abb1a0a90885"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "python_package_version_entity_rule",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_name", sa.Text(), nullable=False),
        sa.Column("version_range", sa.Text(), nullable=True),
        sa.Column("python_package_index_id", sa.Integer(), nullable=True),
        sa.Column("description", sa.Text(), nullable=True),
        sa.ForeignKeyConstraint(["python_package_index_id"], ["python_package_index.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("id"),
    )
    op.create_index(
        "python_package_version_entity_rule_package_name_idx",
        "python_package_version_entity_rule",
        ["package_name"],
        unique=False,
    )
    op.create_table(
        "python_package_version_entity_rules_association",
        sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False),
        sa.Column("python_package_version_entity_rule_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_rule_id"], ["python_package_version_entity_rule.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("python_package_version_entity_id", "python_package_version_entity_rule_id"),
    )
    op.create_index(
        "python_entity_rules_association_table_idx",
        "python_package_version_entity_rules_association",
        ["python_package_version_entity_id"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "python_entity_rules_association_table_idx", table_name="python_package_version_entity_rules_association"
    )
    op.drop_table("python_package_version_entity_rules_association")
    op.drop_index(
        "python_package_version_entity_rule_package_name_idx", table_name="python_package_version_entity_rule"
    )
    op.drop_table("python_package_version_entity_rule")
    # ### end Alembic commands ###



"""Add indexes needed for revsolver

Revision ID: a7442899ef28
Revises: ecb0ca258835
Create Date: 2020-04-21 19:19:43.848258+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "a7442899ef28"
down_revision = "ecb0ca258835"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("depends_on_entity_id_idx", "depends_on", ["entity_id"], unique=False)
    op.create_index(
        "python_package_version_environment_idx",
        "python_package_version",
        ["os_name", "os_version", "python_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_entity_package_name_idx",
        "python_package_version_entity",
        ["package_name"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("python_package_version_entity_package_name_idx", table_name="python_package_version_entity")
    op.drop_index("python_package_version_environment_idx", table_name="python_package_version")
    op.drop_index("depends_on_entity_id_idx", table_name="depends_on")
    # ### end Alembic commands ###



"""Add parameters to deal with unsolved pacakges in adviser run

Revision ID: 125a71b49a50
Revises: 74254b596f00
Create Date: 2020-03-23 13:46:21.439043+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "125a71b49a50"
down_revision = "74254b596f00"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("adviser_run", sa.Column("need_re_run", sa.Boolean(), nullable=True))
    op.add_column("adviser_run", sa.Column("re_run_adviser_id", sa.String(length=256), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("adviser_run", "re_run_adviser_id")
    op.drop_column("adviser_run", "need_re_run")
    # ### end Alembic commands ###
